@@ -21,6 +21,7 @@
 
 #include "camel-imapx-conn-manager.h"
 #include "camel-imapx-settings.h"
+#include "camel-imapx-store.h"
 #include "camel-imapx-utils.h"
 
 #define c(...) camel_imapx_debug(conman, __VA_ARGS__)
@@ -38,7 +39,7 @@ G_DEFINE_TYPE (
 	CAMEL_TYPE_OBJECT)
 
 struct _CamelIMAPXConnManagerPrivate {
-	GSList *connections;
+	GList *connections;
 	gpointer store;  /* weak pointer */
 	GStaticRecMutex con_man_lock;
 	gboolean clearing_connections;
@@ -77,7 +78,7 @@ imapx_prune_connections (CamelIMAPXConnManager *con_man)
 	CON_LOCK (con_man);
 
 	con_man->priv->clearing_connections = TRUE;
-	g_slist_foreach (con_man->priv->connections, (GFunc) free_connection, NULL);
+	g_list_foreach (con_man->priv->connections, (GFunc) free_connection, NULL);
 	con_man->priv->connections = NULL;
 	con_man->priv->clearing_connections = FALSE;
 
@@ -207,7 +208,7 @@ static void
 imapx_conn_shutdown (CamelIMAPXServer *conn,
                      CamelIMAPXConnManager *con_man)
 {
-	GSList *l;
+	GList *l;
 	ConnectionInfo *cinfo;
 	gboolean found = FALSE;
 
@@ -222,7 +223,7 @@ imapx_conn_shutdown (CamelIMAPXServer *conn,
 
 	CON_LOCK (con_man);
 
-	for (l = con_man->priv->connections; l != NULL; l = g_slist_next (l)) {
+	for (l = con_man->priv->connections; l != NULL; l = g_list_next (l)) {
 		cinfo = (ConnectionInfo *) l->data;
 		if (cinfo->conn == conn) {
 			found = TRUE;
@@ -231,7 +232,7 @@ imapx_conn_shutdown (CamelIMAPXServer *conn,
 	}
 
 	if (found) {
-		con_man->priv->connections = g_slist_remove (con_man->priv->connections, cinfo);
+		con_man->priv->connections = g_list_remove (con_man->priv->connections, cinfo);
 		free_connection (cinfo, GINT_TO_POINTER (1));
 	}
 
@@ -243,13 +244,13 @@ imapx_conn_update_select (CamelIMAPXServer *conn,
                           const gchar *selected_folder,
                           CamelIMAPXConnManager *con_man)
 {
-	GSList *l;
+	GList *l;
 	ConnectionInfo *cinfo;
 	gboolean found = FALSE;
 
 	CON_LOCK (con_man);
 
-	for (l = con_man->priv->connections; l != NULL; l = g_slist_next (l)) {
+	for (l = con_man->priv->connections; l != NULL; l = g_list_next (l)) {
 		cinfo = (ConnectionInfo *) l->data;
 		if (cinfo->conn == conn) {
 			found = TRUE;
@@ -282,7 +283,7 @@ imapx_find_connection (CamelIMAPXConnManager *con_man,
                        const gchar *folder_name)
 {
 	guint i = 0, prev_len = -1, n = -1;
-	GSList *l;
+	GList *l;
 	CamelIMAPXServer *conn = NULL;
 	CamelService *service;
 	CamelSettings *settings;
@@ -299,7 +300,7 @@ imapx_find_connection (CamelIMAPXConnManager *con_man,
 		CAMEL_IMAPX_SETTINGS (settings));
 
 	/* Have a dedicated connection for INBOX ? */
-	for (l = con_man->priv->connections, i = 0; l != NULL; l = g_slist_next (l), i++) {
+	for (l = con_man->priv->connections, i = 0; l != NULL; l = g_list_next (l), i++) {
 		IMAPXJobQueueInfo *jinfo = NULL;
 
 		cinfo = (ConnectionInfo *) l->data;
@@ -325,8 +326,8 @@ imapx_find_connection (CamelIMAPXConnManager *con_man,
 		}
 	}
 
-	if (!conn && n != -1 && (!folder_name || concurrent_connections == g_slist_length (con_man->priv->connections))) {
-		cinfo = g_slist_nth_data (con_man->priv->connections, n);
+	if (!conn && n != -1 && (!folder_name || concurrent_connections == g_list_length (con_man->priv->connections))) {
+		cinfo = g_list_nth_data (con_man->priv->connections, n);
 		conn = g_object_ref (cinfo->conn);
 
 		if (folder_name) {
@@ -336,7 +337,7 @@ imapx_find_connection (CamelIMAPXConnManager *con_man,
 	}
 
 	if (camel_debug_flag (conman))
-		g_assert (!(concurrent_connections == g_slist_length (con_man->priv->connections) && !conn));
+		g_assert (!(concurrent_connections == g_list_length (con_man->priv->connections) && !conn));
 
 	CON_UNLOCK (con_man);
 
@@ -350,20 +351,46 @@ imapx_create_new_connection (CamelIMAPXConnManager *con_man,
                              GError **error)
 {
 	CamelIMAPXServer *conn;
+	CamelIMAPXStore *imapx_store;
 	CamelStore *store = con_man->priv->store;
 	CamelService *service;
 	CamelURL *url;
 	ConnectionInfo *cinfo = NULL;
+	gboolean success;
 
 	service = CAMEL_SERVICE (store);
 	url = camel_service_get_camel_url (service);
 
+	imapx_store = CAMEL_IMAPX_STORE (store);
+
 	CON_LOCK (con_man);
 
 	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	conn = camel_imapx_server_new (store, url);
-	if (camel_imapx_server_connect (conn, cancellable, error)) {
+
+	/* XXX As part of the connect operation the CamelIMAPXServer will
+	 *     have to call camel_session_authenticate_sync(), but it has
+	 *     no way to pass itself through in that call so the service
+	 *     knows which CamelIMAPXServer is trying to authenticate.
+	 *
+	 *     IMAPX is the only provider that does multiple connections
+	 *     like this, so I didn't want to pollute the CamelSession and
+	 *     CamelService authentication APIs with an extra argument.
+	 *     Instead we do this little hack so the service knows which
+	 *     CamelIMAPXServer to act on in its authenticate_sync() method.
+	 *
+	 *     Because we're holding the CAMEL_SERVICE_REC_CONNECT_LOCK
+	 *     (and our own CON_LOCK for that matter) we should not have
+	 *     multiple IMAPX connections trying to authenticate at once,
+	 *     so this should be thread-safe.
+	 */
+	imapx_store->authenticating_server = g_object_ref (conn);
+	success = camel_imapx_server_connect (conn, cancellable, error);
+	g_object_unref (imapx_store->authenticating_server);
+	imapx_store->authenticating_server = NULL;
+
+	if (success) {
 		g_object_ref (conn);
 	} else {
 		g_object_unref (conn);
@@ -386,9 +413,9 @@ imapx_create_new_connection (CamelIMAPXConnManager *con_man,
 	if (folder_name)
 		g_hash_table_insert (cinfo->folders, g_strdup (folder_name), GINT_TO_POINTER (1));
 
-	con_man->priv->connections = g_slist_prepend (con_man->priv->connections, cinfo);
+	con_man->priv->connections = g_list_prepend (con_man->priv->connections, cinfo);
 
-	c(conn->tagprefix, "Created new connection for %s and total connections %d \n", folder_name, g_slist_length (con_man->priv->connections));
+	c(conn->tagprefix, "Created new connection for %s and total connections %d \n", folder_name, g_list_length (con_man->priv->connections));
 
 	CON_UNLOCK (con_man);
 
@@ -435,17 +462,17 @@ camel_imapx_conn_manager_get_connection (CamelIMAPXConnManager *con_man,
 	return conn;
 }
 
-GSList *
+GList *
 camel_imapx_conn_manager_get_connections (CamelIMAPXConnManager *con_man)
 {
-	GSList *l, *conns = NULL;
+	GList *l, *conns = NULL;
 
 	CON_LOCK (con_man);
 
-	for (l = con_man->priv->connections; l != NULL; l = g_slist_next (l)) {
+	for (l = con_man->priv->connections; l != NULL; l = g_list_next (l)) {
 		ConnectionInfo *cinfo = (ConnectionInfo *) l->data;
 
-		conns = g_slist_prepend (conns, g_object_ref (cinfo->conn));
+		conns = g_list_prepend (conns, g_object_ref (cinfo->conn));
 	}
 
 	CON_UNLOCK (con_man);
@@ -459,7 +486,7 @@ camel_imapx_conn_manager_update_con_info (CamelIMAPXConnManager *con_man,
                                           CamelIMAPXServer *conn,
                                           const gchar *folder_name)
 {
-	GSList *l;
+	GList *l;
 	ConnectionInfo *cinfo;
 	gboolean found = FALSE;
 
@@ -467,7 +494,7 @@ camel_imapx_conn_manager_update_con_info (CamelIMAPXConnManager *con_man,
 
 	CON_LOCK (con_man);
 
-	for (l = con_man->priv->connections; l != NULL; l = g_slist_next (l)) {
+	for (l = con_man->priv->connections; l != NULL; l = g_list_next (l)) {
 		cinfo = (ConnectionInfo *) l->data;
 		if (cinfo->conn == conn) {
 			found = TRUE;
